function outtable = parseSparkyDecayList(fname, fac)
% parseSparkyDecayList  
%
%  This function reads the residue numbers and intensities stored in a 
%  Sparky exponential decay fitting list.  The T1 decay and std. dev.
%  fields are ignored.
%
%  Synopsis:
%     [labels, times, heights] = parseSparkyDecayList(fname)
%     [labels, times, heights] = parseSparkyDecayList(fname, fac)
%
%  Input:
%     fname = Name of the file to be read. (required)
%     fac   = A factor for converting the times to seconds. 
%             Default = 0.001 (milliseconds)
%
%  Output:
%     labels  = List of residue numbers, parsed from the first column
%     times   = List of mixing times, in seconds
%     heights = Matrix of peak heights (i, j). heights[i, j] corresponds
%               to the intensitiy of peak label[i] at time times[j].
%
%  We guarantee that the labels and heights will be sorted in order from
%  lowest residue to highest.
%

if nargin < 2
    fac = 0.001;
end

fh = fopen(fname, 'r');

if fh < 0
    error(['Could not open ', fname, ' for reading.']);
end

header = fgetl(fh);
times  = [];

if header < 0
    error('Cannot parse empty file.');
end

[~, header] = strtok(header); % "Assignment"
[~, header] = strtok(header); % "T-decay"
[~, header] = strtok(header); % "SD"

% -------------------These codes are for get mixing Times
while true                                  
    [tok, header] = strtok(header);
    if isempty(tok) break; end
    [~, time] = strtok(tok, '/');
    times = cat(2, times, str2double(time(2:end))*fac);
end
% -------------------end of get mixing Times

% New hash table for storing/sorting data
tmp = containers.Map('KeyType', 'int32', 'ValueType', 'any');

line = fgetl(fh);

while ischar(line)
    line = strtrim(line);
    %fprintf('%s\n', line);
    
    if isempty(line) || line(1) == '#'
        line = fgetl(fh);
        continue;
    end

    % Parse the line; get data and the residue label
    [label, line] = strtok(line);
    [~, line]     = strtok(line); % Value of T-Decay
    [~, line]     = strtok(line); % Value of SD
    data = transpose(sscanf(line, '%f'));

    % Grab residue number from label string
    retok = regexp(label, '\D(\d+)\D+-.*', 'tokens');
    
    if isempty(retok)
        fprintf('Skipping bad label: %s\n', label);
        line = fgetl(fh);
        continue;
    end
        
    label = str2double(retok{1}{1});
    
    if ~isKey(tmp, label)
        tmp(label) = data;
    else
        fprintf('Skipping duplicate residue number: %i\n', label);
        line = fgetl(fh);
        continue;
    end
    
    line = fgetl(fh);
end

fclose(fh);


% Probably not necessary; MATLAB seems to sort keys automatically,
% but I can't find a guarantee that this is the case.
myKeys = keys(tmp);
labels = sort(cell2mat(myKeys));
heights = [];

verticallabels = transpose(labels);

for i = 1:size(labels,2)
    label = labels(i);
    heights = cat(1, heights, double(   tmp(label)   )  );   % double to get more precision
end

outtable = [double(verticallabels) heights];











